//###########################################################################
//
// FILE:   adc_ex2_epwm_tempsensor.c
//
// TITLE:  Sample temperature sensor and convert to temperature
//
//! \addtogroup bitfield_example_list
//! <h1> ADC temperature sensor conversion</h1>
//!
//! This example sets up the ePWM to periodically trigger the ADC. The
//! ADC converts the internal connection to the temperature sensor,
//! which is then interpreted as a temperature by calling the
//! GetTemperatureC function.
//!
//! After the program runs, the memory will contain:\n
//! - \b sensorSample \b : The raw reading from the temperature sensor. \n
//! - \b sensorTemp \b : The interpretation of the sensor sample as a
//! temperature in degrees Celsius.
//!
//
//#############################################################################
//
//
// 
// C2000Ware v5.04.00.00
//
// Copyright (C) 2024 Texas Instruments Incorporated - http://www.ti.com
//
// Redistribution and use in source and binary forms, with or without 
// modification, are permitted provided that the following conditions 
// are met:
// 
//   Redistributions of source code must retain the above copyright 
//   notice, this list of conditions and the following disclaimer.
// 
//   Redistributions in binary form must reproduce the above copyright
//   notice, this list of conditions and the following disclaimer in the 
//   documentation and/or other materials provided with the   
//   distribution.
// 
//   Neither the name of Texas Instruments Incorporated nor the names of
//   its contributors may be used to endorse or promote products derived
//   from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// $
//#############################################################################

//
// Included Files
//
#include "f28x_project.h"
#include <stdarg.h> // For va_list, va_start, va_end
#include <stdio.h> // For vsnprintf
//
// Defines
//
#define RESULTS_BUFFER_SIZE     256
#define BUFFER_SIZE 128
#define TIMER_PERIOD 1000000 // wɾgA쬰L
#define PWM_PERIOD 278
//
// Globals
//
uint16_t sensorSample = 0;
uint16_t isrCount = 0;
int16_t sensorTemp = 0;
int16_t ADC1_raw=0;
int16_t ADC2_raw=0;

//
// Function Prototypes
//
void initADC(void);
void initEPWM6(void);
void initEPWM1(void);
void initADCSOC(void);
__interrupt void adcC1ISR(void);
__interrupt void adcB1ISR(void);
__interrupt void adcA1ISR(void);
__interrupt void cpuTimer0ISR(void);
void initSCIAEchoback(void);
void transmitSCIAChar(uint16_t a);
void transmitSCIAMessage(unsigned char * msg);
void initSCIAFIFO(void);

void sciPrintf(const char *format, ...);
void InitTimer(void);

volatile Uint32 timerCount = 0;
//
// Main
//
void main(void)
{
    //
    // Initialize device clock and peripherals
    //
    InitSysCtrl();

    //
    // Initialize GPIO
    //
    InitGpio();
    GPIO_SetupPinMux(28, GPIO_MUX_CPU1, 1);
    GPIO_SetupPinOptions(28, GPIO_INPUT, GPIO_PUSHPULL);
    GPIO_SetupPinMux(29, GPIO_MUX_CPU1, 1);
    GPIO_SetupPinOptions(29, GPIO_OUTPUT, GPIO_ASYNC);
    initSCIAFIFO();                         // Initialize the SCI FIFO
    initSCIAEchoback();                     // Initialize SCI for echoback

    sciPrintf("\r\n\n\n ADC Enabler! Test V001 \0");

    InitTimer(); // lƩwɾ
    //
    // Disable CPU interrupts
    //
    DINT;


    InitPieCtrl();

    //
    // Disable CPU interrupts and clear all CPU interrupt flags:
    //
    IER = 0x0000;
    IFR = 0x0000;

    //
    // Initialize the PIE vector table with pointers to the shell Interrupt
    // Service Routines (ISR).
    //
    InitPieVectTable();

    //
    // Map ISR functions
    //
    EALLOW;
    PieVectTable.TIMER0_INT = &cpuTimer0ISR;
    PieVectTable.ADCC1_INT = &adcC1ISR;     // Function for ADCC interrupt 1
    PieVectTable.ADCB1_INT = &adcB1ISR;
    PieVectTable.ADCA1_INT = &adcA1ISR;
    EDIS;

    //
    // Configure the ADC and power it up
    //
    initADC();

    //
    // Configure the ePWM
    //
    initEPWM1();
    PieCtrlRegs.PIEIER1.bit.INTx3 = 1;
    //
    // Setup the ADC for ePWM triggered conversions on channel 1
    //
    initADCSOC();


    EALLOW;


    //CpuSysRegs.PCLKCR0.bit.TBCLKSYNC = 1;
    InitCpuTimers();
    ConfigCpuTimer(&CpuTimer0, 150, 1000000); // 150MHz CPU Freq, 1 second Period (in uSeconds)
    CpuTimer0Regs.TCR.all = 0x4000;

    //
    // Enable global Interrupts and higher priority real-time debug events:
    //
    IER |= M_INT1;  // Enable group 1 interrupts
    //IER |= M_INT10;  // Enable group 1 interrupts
    PieCtrlRegs.PIEIER1.bit.INTx7 = 1;
    PieCtrlRegs.PIEIER1.bit.INTx3 = 1;
    PieCtrlRegs.PIEIER1.bit.INTx2 = 1; // ҥβ 1  ADCB _
    PieCtrlRegs.PIEIER1.bit.INTx1 = 1; // ҥβ 1  ADCA _
    EINT;           // Enable Global interrupt INTM
    ERTM;           // Enable Global realtime interrupt DBGM

    //InitTempSensor(3.3f);

    //
    // Start ePWM
    //
    EPwm1Regs.ETSEL.bit.SOCAEN = 1;    // Enable SOCA
    EPwm1Regs.TBCTL.bit.CTRMODE = 2;   // Unfreeze, and enter up count mode
    //EPwm6Regs.ETSEL.bit.SOCAEN = 1;    // Enable SOCA
     //   EPwm6Regs.TBCTL.bit.CTRMODE = 0;   // Unfreeze, and enter up count mode

    //
    // Wait while ePWM causes ADC conversions, which then cause interrupts,
    // which fill the results buffer, eventually setting the bufferFull
    // flag
    //
    while(1)
    {

    }
}

//
// initADC - Function to configure and power up ADCC.
//
void initADC(void)
{
    //
    // Setup VREF as internal
    //
    SetVREF(ADC_ADCC, ADC_INTERNAL, ADC_VREF2P5);
    SetVREF(ADC_ADCB, ADC_INTERNAL, ADC_VREF2P5);
    SetVREF(ADC_ADCA, ADC_INTERNAL, ADC_VREF2P5);
    EALLOW;

    //
    // Set ADCCLK divider to /4
    //
    AdccRegs.ADCCTL2.bit.PRESCALE = 6;
    AdcbRegs.ADCCTL2.bit.PRESCALE = 6;
    AdcaRegs.ADCCTL2.bit.PRESCALE = 6;


    //
    // Set pulse positions to late EOC
    //
    AdccRegs.ADCCTL1.bit.INTPULSEPOS = 1;
    AdcbRegs.ADCCTL1.bit.INTPULSEPOS = 1;
    AdcaRegs.ADCCTL1.bit.INTPULSEPOS = 1;
    //
    // Power up the ADC and then delay for 1 ms
    //
    AdccRegs.ADCCTL1.bit.ADCPWDNZ = 1;
    AdcbRegs.ADCCTL1.bit.ADCPWDNZ = 1;
    AdcaRegs.ADCCTL1.bit.ADCPWDNZ = 1;
    EDIS;

    DELAY_US(1000);
}

//
// initEPWM - Function to configure ePWM1 to generate the SOC.
//
void initEPWM6(void)
{
    EALLOW;
    CpuSysRegs.PCLKCR2.bit.EPWM1 = 1;
    //tm PWM ޸}
    InitEPwm1Gpio();
           // tm EPWM1 Ѽ
     EPwm1Regs.TBPRD = 30000;//PWM_PERIOD - 1; // PWM g
     EPwm1Regs.TBCTL.bit.CTRMODE = TB_COUNT_UPDOWN;//TB_COUNT_UP; // ]wVWpƼҦ


           EPwm1Regs.AQCTLA.bit.ZRO = AQ_CLEAR;

    EPwm6Regs.ETSEL.bit.SOCAEN = 0;     // Disable SOC on A group
    EPwm6Regs.ETSEL.bit.SOCASEL = 4;    // Select SOC on up-count
    EPwm6Regs.ETPS.bit.SOCAPRD = 1;     // Generate pulse on 1st event

    //EPwm6Regs.CMPA.bit.CMPA = 0x0800;   // Set compare A value to 2048 counts
   // EPwm6Regs.TBPRD = 0x1000;           // Set period to 4096 counts

   // EPwm6Regs.TBCTL.bit.CTRMODE = 3;    // Freeze counter

    EDIS;
}
void initEPWM1(void)
{
    EALLOW;
    CpuSysRegs.PCLKCR2.bit.EPWM1 = 1;
    InitEPwm1Gpio();

    EPwm1Regs.TBPRD = PWM_PERIOD - 1; // PWM g
    EPwm1Regs.TBCTL.bit.CTRMODE = TB_COUNT_UP;//TB_COUNT_UPDOWN;//TB_COUNT_UP; // ]wVWpƼҦ
    EPwm1Regs.CMPA.bit.CMPA = 100;
    EPwm1Regs.AQCTLA.bit.CAU = AQ_CLEAR;//AQ_SET; // pƾ CMPA ɳ]m PWM H
    EPwm1Regs.AQCTLA.bit.CAD = AQ_SET;//AQ_CLEAR; // pƾ PWM gɲM PWM H

    EPwm1Regs.ETSEL.bit.SOCAEN = 1;     // Disable SOC on A group
    EPwm1Regs.ETSEL.bit.SOCASEL = 4;    // Select SOC on up-count
    EPwm1Regs.ETPS.bit.SOCAPRD = 1;     // Generate pulse on 1st event


    EDIS;
}

//
// initADCSOC - Function to configure ADCA's SOC0 to be triggered by ePWM1.
//
void initADCSOC(void)
{
    //
    // Select the channels to convert and the end of conversion flag
    //
    EALLOW;

    AdccRegs.ADCSOC0CTL.bit.CHSEL = 12;     // SOC0 will convert pin C12
    AdccRegs.ADCSOC0CTL.bit.ACQPS = 63;//45;     // Sample window is 10 SYSCLK cycles
    AdccRegs.ADCSOC0CTL.bit.TRIGSEL = 5;   // Trigger on ePWM1 SOCA
    AdccRegs.ADCINTSEL1N2.bit.INT1SEL = 0; // End of SOC0 will set INT1 flag
    AdccRegs.ADCINTSEL1N2.bit.INT1E = 1;   // Enable INT1 flag
    AdccRegs.ADCINTFLGCLR.bit.ADCINT1 = 1; // Make sure INT1 flag is cleared
    AdccRegs.ADCSOC0CTL.bit.SAMPCAPRESETDISABLE = 0;

    // tm ADCB SOC0 ഫM_
    AdcbRegs.ADCSOC0CTL.bit.CHSEL = 0;
    AdcbRegs.ADCSOC0CTL.bit.ACQPS = 63;//45;
    AdcbRegs.ADCSOC0CTL.bit.TRIGSEL = 5;
    AdcbRegs.ADCINTSEL1N2.bit.INT1SEL = 0;
    AdcbRegs.ADCINTSEL1N2.bit.INT1E = 1;
    AdcbRegs.ADCINTFLGCLR.bit.ADCINT1 = 1;
    AdcbRegs.ADCSOC0CTL.bit.SAMPCAPRESETDISABLE = 0;
    // tm ADCA SOC0 ഫM_
    AdcaRegs.ADCSOC0CTL.bit.CHSEL = 10;
    AdcaRegs.ADCSOC0CTL.bit.ACQPS = 63;//45;
    AdcaRegs.ADCSOC0CTL.bit.TRIGSEL = 5;
    AdcaRegs.ADCINTSEL1N2.bit.INT1SEL = 0;
    AdcaRegs.ADCINTSEL1N2.bit.INT1E = 1;
    AdcaRegs.ADCINTFLGCLR.bit.ADCINT1 = 1;
    AdcaRegs.ADCSOC0CTL.bit.SAMPCAPRESETDISABLE = 0;
    EDIS;
}

//
// adcC1ISR - ADC C Interrupt 1 ISR
//
__interrupt void adcC1ISR(void)
{

    //isrCount++;

    sensorSample  = AdccResultRegs.ADCRESULT0;
    sensorTemp = GetTemperatureC(sensorSample);


    AdccRegs.ADCINTFLGCLR.bit.ADCINT1 = 1;

    if(1 == AdccRegs.ADCINTOVF.bit.ADCINT1)
    {
        //sciPrintf("ADC C buffer full\r\n");
        AdccRegs.ADCINTOVFCLR.bit.ADCINT1 = 1; //clear INT1 overflow flag
        AdccRegs.ADCINTFLGCLR.bit.ADCINT1 = 1; //clear INT1 flag
    }

    PieCtrlRegs.PIEACK.all = PIEACK_GROUP1;
}
__interrupt void adcB1ISR(void)
{

    //isrCount++;

    ADC2_raw  = AdcbResultRegs.ADCRESULT0;
    //sensorTemp = GetTemperatureC(sensorSample);


    AdcbRegs.ADCINTFLGCLR.bit.ADCINT1 = 1;

    if(1 == AdcbRegs.ADCINTOVF.bit.ADCINT1)
    {
       // sciPrintf("ADC B buffer full\r\n");
        AdcbRegs.ADCINTOVFCLR.bit.ADCINT1 = 1; //clear INT1 overflow flag
        AdcbRegs.ADCINTFLGCLR.bit.ADCINT1 = 1; //clear INT1 flag
    }

    PieCtrlRegs.PIEACK.all = PIEACK_GROUP1;
}
__interrupt void adcA1ISR(void)
{

    //isrCount++;

    ADC1_raw  = AdcaResultRegs.ADCRESULT0;
    //sensorTemp = GetTemperatureC(sensorSample);


    AdcaRegs.ADCINTFLGCLR.bit.ADCINT1 = 1;

    if(1 == AdcaRegs.ADCINTOVF.bit.ADCINT1)
    {
        //sciPrintf("ADC A buffer full\r\n");
        AdcaRegs.ADCINTOVFCLR.bit.ADCINT1 = 1; //clear INT1 overflow flag
        AdcaRegs.ADCINTFLGCLR.bit.ADCINT1 = 1; //clear INT1 flag
    }

    PieCtrlRegs.PIEACK.all = PIEACK_GROUP1;
}
void initSCIAEchoback(void)
{
    //
    // Note: Clocks were turned on to the SCIA peripheral
    // in the InitSysCtrl() function
    //
    SciaRegs.SCICCR.all = 0x0007;           // 1 stop bit,  No loopback
                                            // No parity, 8 char bits,
                                            // async mode, idle-line protocol
    SciaRegs.SCICTL1.all = 0x0003;          // enable TX, RX, internal SCICLK,
                                            // Disable RX ERR, SLEEP, TXWAKE
    SciaRegs.SCICTL2.all = 0x0003;
    SciaRegs.SCICTL2.bit.TXINTENA = 1;
    SciaRegs.SCICTL2.bit.RXBKINTENA = 1;

    //
    // SCIA at 9600 baud
    // @LSPCLK = 37.5 MHz (150 MHz SYSCLK) HBAUD = 0x01 and LBAUD = 0xE7.
    //
    SciaRegs.SCIHBAUD.all = 0x0001;
    SciaRegs.SCILBAUD.all = 0x00E7;

    SciaRegs.SCICTL1.all = 0x0023;          // Relinquish SCI from Reset
}

//
// transmitSCIAChar - Transmit a character from the SCI
//
void transmitSCIAChar(uint16_t a)
{
    while (SciaRegs.SCIFFTX.bit.TXFFST != 0)
    {

    }
    SciaRegs.SCITXBUF.all = a;
}

//
// transmitSCIAMessage - Transmit message via SCIA
//
void transmitSCIAMessage(unsigned char * msg)
{
    int i;
    i = 0;
    while(msg[i] != '\0')
    {
        transmitSCIAChar(msg[i]);
        i++;
    }
}

//
// initSCIAFIFO - Initialize the SCI FIFO
//
void initSCIAFIFO(void)
{
    SciaRegs.SCIFFTX.all = 0xE040;
    SciaRegs.SCIFFRX.all = 0x2044;
    SciaRegs.SCIFFCT.all = 0x0;
}
void sciPrintf(const char *format, ...)
{
    char buffer[BUFFER_SIZE];
    int i;
    va_list args;
    va_start(args, format);
    vsnprintf(buffer, BUFFER_SIZE, format, args);
    va_end(args);
    for (i = 0; buffer[i] != '\0'; i++)
    {
        transmitSCIAChar(buffer[i]);
    }
}
void InitTimer(void)
{
    sciPrintf("InitTimer\r\n");
    ConfigCpuTimer(&CpuTimer0, 200, TIMER_PERIOD);
    CpuTimer0Regs.TCR.all = 0x4000;
}


__interrupt void cpuTimer0ISR(void)
{


    sciPrintf("ADC C value= %d\r\n",sensorSample);
    sciPrintf("ADC A value= %d\r\n",ADC1_raw);
    sciPrintf("ADC B value= %d\r\n",ADC2_raw);
    sensorSample=0;
    ADC1_raw=0;
    ADC2_raw=0;

   // sciPrintf("Temperature= %d\r\n",sensorTemp);


    CpuTimer0Regs.TCR.bit.TIF = 1;

    PieCtrlRegs.PIEACK.all = PIEACK_GROUP1;
}
//
// End of file
//
